/*****************************/
/* 7章 常微分方程式の多段解法 */
/*****************************/

/* 本書の例題7.2に対する4点を用いるAdams法 */
/* （Adams-BashforthとAdams-Moultonによる予測子・修正子法） */

#include<stdio.h>
#include<math.h>

#define M 2 // 検証する刻み幅の個数
#define K 3 // 段数-1（4点を用いるアダムズ法）

/* 1階導関数 */
double f(double x, double y){
  return y-x;
}

/* 解析解 */
double y(double x){
  return x+1-pow(M_E,x);
}

/* メイン関数 */
int main(void){
  int i, n, N;
  int num[M] = {2,3};
  double h, x0, xn, k1, k2, k3, k4;
  double x[(int)(pow(2,num[M-1]))+1]; // 分割点（最大分割数+1 の領域を確保） 
  double Y[(int)(pow(2,num[M-1]))+1]; // 近似解（最大分割数+1 の領域を確保） 

  x0 = 0.0; // 始点
  xn = 1.0; // 終点

  printf("%6s: %19s: %10s\n", "h", "Y(1)", "|Y(1)-y(1)|");
  for(i=0; i<40; i++){
	printf("-");
  }
  printf("\n");
  
  /* 各刻み幅に対して検証 */
  for(i=0; i<M; i++){

	N = pow(2,num[i]); // 分割数
	h = (xn - x0)/N;   // 刻み幅

	x[0] = x0;  // 点の初期化
	Y[0] = 0.0; // 初期条件

	/* ルンゲ・クッタ法による予備計算（Y1,Y2,Y3を求める） */
	for(n=0; n<K; n++){
	  k1 = f(x[n],Y[n]);
	  x[n+1] = x[n] + h/2.0;
	  k2 = f(x[n+1],Y[n]+h*k1/2.0);
	  k3 = f(x[n+1],Y[n]+h*k2/2.0);
	  x[n+1] = x[n+1] + h/2.0;
	  k4 = f(x[n+1],Y[n]+h*k3);
	  Y[n+1] = Y[n] + h*(k1 + 2.0*k2 + 2.0*k3 + k4)/6.0;
	}

	/* アダムズ法の計算 */
	for(n=K; n<N; n++){
	  /* 予測子（アダムズ・バシュフォース法） */
	  Y[n+1] = Y[n] + h*(55*f(x[n],Y[n]) - 59*f(x[n-1],Y[n-1])
						 + 37*f(x[n-2],Y[n-2]) - 9*f(x[n-3],Y[n-3]))/24.0;
	  x[n+1] = x[n] + h;
	  /* 修正子（アダムズ・モールトン法） */
	  Y[n+1] = Y[n] + h*(9*f(x[n+1],Y[n+1]) + 19*f(x[n],Y[n])
						 - 5*f(x[n-1],Y[n-1]) + f(x[n-2],Y[n-2]))/24.0;
	}
	
	/* 計算結果の出力 */
	printf("2^{-%d}: %.16f: %.2e\n", num[i], Y[n], fabs(Y[n]-y(1)));
	
  }

  return 0;
}
