/****************************/
/* 8章 連立方程式の反復解法  */
/****************************/

/* 本書の演習問題8.7[8.2-(ii)]に対するCG法（相対残差判定） */
/* 疎行列形式（CRS形式）を使用 */

#include<stdio.h>
#include<math.h>

#define N 10       // 行列の次元
#define NNZ 28     // 非ゼロ要素数
#define EPS 1.0e-9 // 許容誤差
#define MMAX 30    // 最大反復回数

/* 行列ベクトル積を計算する関数 */
void mat_vec(double y[], double val[], int col_ind[], int row_ptr[], double x[]){
  int i, j;
  for(i=0; i<N; i++){
	y[i] = 0;
	for(j=row_ptr[i]; j<row_ptr[i+1]; j++){
	  y[i] += val[j]*x[col_ind[j]];
	}
  }
  return;
}

/* ベクトルの更新(AXPY)を計算する関数 */
void axpy(double z[], double y[], double a, double x[]){
  int i;
  for(i=0; i<N; i++){
	z[i] = y[i] + a*x[i];
  }
  return;
}

/* ベクトル同士の内積を計算する関数 */
double dot(double x[], double y[]){
  int i;
  double z = 0.0;
  for(i=0; i<N; i++){
	z += x[i]*y[i];
  }
  return z;
}

/* メイン関数 */
int main(void){
  int i, j, m;
  double b_2, alpha, beta, sigma, rho_old, rho_new;

  /* 係数行列（CRS形式） */
  double val[NNZ] = {2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0};
  int col_ind[NNZ] = {0,1,0,1,2,1,2,3,2,3,4,3,4,5,4,5,6,5,6,7,6,7,8,7,8,9,8,9};
  int row_ptr[N+1] = {0,2,5,8,11,14,17,20,23,26,28};
  
  double b[N] = {1,-1,2,-1,-1,2,-2,2,-1,1}; // 右辺ベクトル
  double x[N]; // 近似解ベクトル
  double r[N]; // 残差ベクトル
  double p[N]; // 探索方向ベクトル
  double w[N]; // 汎用ベクトル

  /* 初期近似解の設定 */
  for(i=0; i<N; i++){
	x[i] = 0.0;
  }

  /* 初期残差の計算 */
  mat_vec(w,val,col_ind,row_ptr,x);
  for(i=0; i<N; i++){
	r[i] = b[i] - w[i];
  }

  /* 初期探索方向の設定 */
  for(i=0; i<N; i++){
	p[i] = r[i];
  }

  /* 右辺ベクトルの2ノルムを保存 */
  b_2 = sqrt(dot(b,b));

  /* 残差の内積を保持 */
  rho_old = dot(r,r);

  /* CG法の反復 */
  m = 0;
  while(1){

	/* 係数alphaの計算 */
	mat_vec(w,val,col_ind,row_ptr,p);
	sigma = dot(p,w);
	alpha = rho_old/sigma;

	/* 近似解の更新 */
	axpy(x,x,alpha,p);

	/* 残差の更新 */
	axpy(r,r,-alpha,w);

	/* 反復回数のカウント */
	m++;
	
	/* 収束判定 */
	if(sqrt(dot(r,r)) < EPS * b_2){
	  printf(" *** 収束しました ***\n");
	  break;
	}else if(m == MMAX){
	  printf(" *** 最大反復回数に到達 ***\n");
	  break;
	}

	/* 係数betaの計算 */
	rho_new = dot(r,r);
	beta = rho_new/rho_old;
	rho_old = rho_new;

	/* 探索方向の更新 */
	axpy(p,r,beta,p);

  }
  
  /* 計算結果の出力 */
  printf("反復回数：%d\n", m);
  printf("近似解ベクトル：\n");
  for(i=0; i<N; i++){
	printf("x[%d] = %.16f\n", i+1, x[i]);
  }
  
  return 0;
}
