/*********************/
/* 9章 固有値の計算  */
/*********************/

/* 本書の例題9.1に対するJacobi法 */
/* 固定回数（収束判定は設定していない） */

#include<stdio.h>
#include<math.h>

#define N 3    // 行列の次元
#define ITER 3 // 反復回数

/* 行列を表示する関数 */
void print_mat(double A[][N]){
  int i, j;
  for(i=0; i<N; i++){
	for(j=0; j<N; j++){
	  printf("%.16f\t", A[i][j]);
	}
	printf("\n");
  }
  printf("\n");
  return;
}

/* メイン関数 */
int main(void){
  int i, j, k, m, q;
  double max, s, c, u, v;

  double A[N][N] = {{3, 0.01, 0.1},
					{0.01, 2, 0.01},
					{0.1, 0.01, 1}};
  double A1[N][N], P[N][N], P1[N][N];

  printf("A = \n");
  print_mat(A);

  /* 固有ベクトル行列の初期化 */
  for(i=0; i<N; i++){
	for(j=0; j<N; j++){
	  if(i == j){
		P[i][j] = 1.0;
	  }else{
		P[i][j] = 0.0;
	  }
	}
  }

  /* ヤコビ法の反復 */
  q = 0;
  while(q < ITER){

	for(i=0; i<66; i++){
	  printf("-");
	}
	printf("\n");
  
	/* ステップ1 */
	max = 0.0;
	for(i=0; i<N-1; i++){
	  for(j=i+1; j<N; j++){
		if(fabs(A[i][j]) > max){
		  max = fabs(A[i][j]);
		  k = i;
		  m = j;
		}
	  }
	}
	printf("k = %d, m = %d\n", k, m);

	/* ステップ2 */
	u = (A[k][k] - A[m][m])/2.0;
	v = sqrt(u*u + A[k][m]*A[k][m]);
	if(u >= 0){
	  c = sqrt((1+u/v)/2.0);
	  s = A[k][m]/(2.0*v*c);
	}else{
	  c = sqrt((1-u/v)/2.0);
	  s = -A[k][m]/(2.0*v*c);
	}
	printf("cos = %.16f, sin = %.16f\n\n", c, s);

	/* ステップ3,4 （回転行列は陽に作らない） */
	for(i=0; i<N; i++){

	  /* 第k行の計算 */
	  if(i == k){
		for(j=i; j<N; j++){ // 上三角部分のみ計算
		  if(j == k){
			A1[i][j] = c*c*A[k][k] + 2.0*s*c*A[k][m] + s*s*A[m][m];
		  }else if(j == m){
			A1[i][j] = 0.0;
		  }else{
			A1[i][j] = c*A[k][j] + s*A[m][j];
		  }
		}
	  }

	  /* 第m行の計算 */
	  else if(i == m){
		for(j=i; j<N; j++){ // 上三角部分のみ計算
		  if(j == k){
			A1[i][j] = 0.0;
		  }else if(j == m){
			A1[i][j] = s*s*A[k][k] - 2.0*s*c*A[k][m] + c*c*A[m][m];
		  }else{
			A1[i][j] = -s*A[k][j] + c*A[m][j];
		  }
		}
	  }

	  /* その他の行の計算 */
	  else{
		for(j=i; j<N; j++){ // 上三角部分のみ計算
		  if(j == k){
			A1[i][j] = c*A[i][k] + s*A[i][m];
		  }else if(j == m){
			A1[i][j] = -s*A[i][k] + c*A[i][m];
		  }else{
			A1[i][j] = A[i][j];
		  }
		}
	  }

	}

	/* 狭義下三角部分にコピー */
	for(i=0; i<N-1; i++){
	  for(j=i+1; j<N; j++){
		A1[j][i] = A1[i][j];
	  }
	}

	/* 固有ベクトル行列の更新 */
	for(i=0; i<N; i++){
	  for(j=0; j<N; j++){
		if(j == k){
		  P1[i][j] = c*P[i][k] + s*P[i][m];
		}else if(j == m){
		  P1[i][j] = -s*P[i][k] + c*P[i][m];
		}else{
		  P1[i][j] = P[i][j];
		}
	  }
	}

	/* 反復回数のカウント */
	q++;

	/* 更新後の行列の表示 */
	printf("A_%d = \n", q);
	print_mat(A1);
	printf("P_0...P_%d = \n", q-1);
	print_mat(P1);

	/* 行列の入れ替え */
	for(i=0; i<N; i++){
	  for(j=0; j<N; j++){
		A[i][j] = A1[i][j];
		P[i][j] = P1[i][j];
	  }
	}
	
  }
  
  return 0;
}
