/****************************/
/* 8章 連立方程式の反復解法  */
/****************************/

/* 本書の演習問題8.2-(ii)に対するJacobi法 */
/* 疎行列形式（CRS形式）を使用 */

#include<stdio.h>
#include<math.h>

#define N 10       // 行列の次元
#define NNZ 28     // 非ゼロ要素数
#define EPS 1.0e-9 // 許容誤差
#define MMAX 500   // 最大反復回数

/* CRS形式の対角成分を取り出す関数 */
void mat_diag(double diag[], double val[], int col_ind[], int row_ptr[]){
  int i, j;
  for(i=0; i<N; i++){
	for(j=row_ptr[i]; j<row_ptr[i+1]; j++){
	  if(i == col_ind[j]){
		diag[i] = val[j];
	  }
	}
  }
  return;
}

/* ベクトルの∞-ノルムを求める関数 */
double norm_inf(double x[]){
  int i;
  double max;
  max = fabs(x[0]);
  for(i=1; i<N; i++){
	if(max < fabs(x[i])){
	  max = fabs(x[i]);
	}
  }
  return max;
}

/* メイン関数 */
int main(void){
  int i, j, m;
  double sum;

  /* 係数行列（CRS形式） */
  double val[NNZ] = {2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0};
  int col_ind[NNZ] = {0,1,0,1,2,1,2,3,2,3,4,3,4,5,4,5,6,5,6,7,6,7,8,7,8,9,8,9};
  int row_ptr[N+1] = {0,2,5,8,11,14,17,20,23,26,28};

  double b[N] = {1,-1,2,-1,-1,2,-2,2,-1,1}; // 右辺ベクトル
  double diag[N];  // 対角成分
  double x[N];     // 近似解ベクトル
  double x_old[N]; // 更新前の近似解ベクトル
  double error[N]; // 近似解の差分を表す誤差ベクトル

  /* 対角成分の取り出し */
  mat_diag(diag, val, col_ind, row_ptr);
  
  /* 初期近似解の設定 */
  for(i=0; i<N; i++){
	x[i] = 0.0;
  }
  
  /* Jacobi法の反復 */
  m = 0;
  while(1){

	/* 近似解の保存 */
	for(i=0; i<N; i++){
	  x_old[i] = x[i];
	}

	/* 近似解の更新 */
	for(i=0; i<N; i++){
	  sum = 0.0;
	  for(j=row_ptr[i]; j<row_ptr[i+1]; j++){
		if(i != col_ind[j]){
		  sum += val[j]*x_old[col_ind[j]];
		}
	  }
	  x[i] = (b[i] - sum)/diag[i];
	}

	/* 反復回数のカウント */
	m++;

	/* 収束判定 */
	for(i=0; i<N; i++){
	  error[i] = x[i] - x_old[i];
	}
	if(norm_inf(error) < EPS * norm_inf(x_old)){
	  printf(" *** 収束しました ***\n");
	  break;
	}else if(m == MMAX){
	  printf(" *** 最大反復回数に到達 ***\n");
	  break;
	}

  }
  
  /* 計算結果の出力 */
  printf("反復回数：%d\n", m);
  printf("近似解ベクトル：\n");
  for(i=0; i<N; i++){
	printf("x[%d] = %.16f\n", i+1, x[i]);
  }
  
  return 0;
}
