/*****************************/
/* 7章 常微分方程式の多段解法 */
/*****************************/

/* 本書の例題7.3に対するLeapfrog法（中点公式） */

#include<stdio.h>
#include<math.h>

/* 1階導関数 */
double f(double x, double y){
  return -y;
}

/* 解析解 */
double y(double x){
  return pow(M_E,-x);
}

/* メイン関数 */
int main(void){
  int i, n, N;
  double h, x, x0, xn;
  double Y[101]; // 分割数+1 の領域を確保

  x0 = 0.0;  // 始点
  xn = 10.0; // 終点

  printf("%3s:\t %18s:\t %18s:\t %10s\n",
		 "x", "Y(x)", "y(x)", "|Y(x)-y(x)|");
  for(i=0; i<70; i++){
	printf("-");
  }
  printf("\n");
  
  N = 100;         // 分割数
  h = (xn - x0)/N; // 刻み幅

  x = x0;     // 点の初期化
  Y[0] = 1.0; // 初期条件

  /* ホイン法による予備計算（A:=-1） */
  Y[1] = (1.0 - h + h*h/2.0)*Y[0];

  /* 中点公式の計算 */
  for(n=1; n<N; n++){
	Y[n+1] = Y[n-1] - 2.0*h*Y[n];
  }

  /* 計算結果の出力 */
  for(n=0; n<=N; n++){
    printf("%.1f:\t %.16f:\t %.16f:\t %.2e\n",
		   x, Y[n], y(x), fabs(Y[n]-y(x)));
	x += h;
  }
  
  return 0;
}
