/*********************************/
/* 3章 非線形方程式に対する反復法 */
/********************************/

/* 本書の例3.2に対するニュートン法（相対誤差判定） */

#include<stdio.h>
#include<math.h>

#define EPS 1.0e-12 // 許容誤差
#define NMAX 20     // 最大反復回数

/* 関数値の計算 */
double f(double x){
  return x - 1.0/exp(x);
}

/* 微分係数の計算 */
double df(double x){
  return 1.0 + 1.0/exp(x);
}

/* メイン関数 */
int main(void){
  int n=0;         // 反復回数
  double x, x_old; // 近似解, 更新前の近似解

  /* 初期近似解の設定 */
  x = 0.5;

  printf(" n: x_n\n");
  printf("----------------------\n");
  printf("%2d: %.16f\n", n, x);

  /* ニュートン反復 */
  while(1){

	/* 近似解の保存 */
	x_old = x;

	/* 近似解の更新 */
	x = x - f(x)/df(x);

	/* 反復回数のカウント */
	n++;

	/* 近似解の出力 */
	printf("%2d: %.16f\n", n, x);

	/* 収束判定 */
	if(fabs(x-x_old) < EPS * fabs(x_old)){
	  printf(" *** 収束しました ***\n");
	  break;
	}else if(n == NMAX){
	  printf(" *** 最大反復回数に到達 ***\n");
	  break;
	}
	
  }

  return 0;
}
