/*******************************/
/* 2章 連立1次方程式の直接解法  */
/*******************************/

/* 本書の演習問題2.1に対する部分ピボット選択付きGaussの消去法 */

#include<stdio.h>
#include<math.h>

#define N 4         // 行列の次元
#define EPS 1.0e-15 // ピボットの許容スケール

/* メイン関数 */
int main(void){
  int i, j, k, sigma;
  double alpha, pivot, tmp, sum;

  double A[N][N] = {{1, 2, -12, 8},
					{5, 4, 7, -2},
					{-3, 7, 9, 5},
					{6, -12, -9, 3}}; // 係数行列
  double b[N] = {27, 4, 11, 49};      // 右辺ベクトル
  double x[N];                        // 解ベクトル

  /* 前進消去過程 */
  for(k=0; k<N-1; k++){

	/* 部分ピボット選択 */
	pivot = fabs(A[k][k]);
	sigma = k;
	for(i=k+1; i<N; i++){
	  if(pivot < fabs(A[i][k])){
		pivot = fabs(A[i][k]);
		sigma = i;
	  }
	}
	if(pivot < EPS){
	  printf("ピボットの値が許容スケール未満です\n");
	  printf(" *** 計算を強制終了します ***\n");
	  break;
	}
	if(k < sigma){ // k行目とsigma行目の交換
	  for(j=k; j<N; j++){
		tmp = A[k][j];
		A[k][j] = A[sigma][j];
		A[sigma][j] = tmp;
	  }
	  tmp = b[k];
	  b[k] = b[sigma];
	  b[sigma] = tmp;
	}

	/* 掃き出し */
	for(i=k+1; i<N; i++){
	  alpha = A[i][k]/A[k][k];
	  for(j=k+1; j<N; j++){
		A[i][j] -= alpha*A[k][j];
	  }
	  b[i] -= alpha*b[k];
	}
	
  }

  /* 後退代入過程 */
  for(i=N-1; i>=0; i--){
	sum = 0;
	for(j=i+1; j<N; j++){
	  sum += A[i][j]*x[j];
	}
	x[i] = (b[i] - sum)/A[i][i];
  }

  /* 近似解の出力 */
  for(i=0; i<N; i++){
	printf("x[%d] = %.16f\n", i+1, x[i]);
  }
  
  return 0;
}
