/*********************/
/* 9章 固有値の計算  */
/*********************/

/* 本書の演習問題9.1-(ii)に対するべき乗法（相対誤差判定） */

#include<stdio.h>
#include<math.h>

#define N 3        // 行列の次元
#define EPS 1.0e-9 // 許容誤差
#define KMAX 30    // 最大反復回数

/* 行列ベクトル積を計算する関数 */
void mat_vec(double y[], double A[][N], double x[]){
  int i, j;
  for(i=0; i<N; i++){
	y[i] = 0;
	for(j=0; j<N; j++){
	  y[i] += A[i][j]*x[j];
	}
  }
  return;
}

/* ベクトル同士の内積を計算する関数 */
double dot(double x[], double y[]){
  int i;
  double z = 0.0;
  for(i=0; i<N; i++){
	z += x[i]*y[i];
  }
  return z;
}

/* ベクトルの2-ノルムを求める関数 */
double norm_2(double x[]){
  int i;
  double z = 0.0;
  for(i=0; i<N; i++){
	z += x[i]*x[i];
  }
  return sqrt(z);
}

/* メイン関数 */
int main(void){
  int i, k, max_i;
  double x_2, max, lam, lam_old;;

  double A[N][N] = {{2,-1,0},
					{-1,2,-1},
					{0,-1,2}};
  double x[N];     // 近似固有ベクトル
  double x_old[N]; // 更新前の近似固有ベクトル
  double y[N];     // レイリー商の計算用

  /* 初期値の設定 */
  lam = 0.0;
  for(i=0; i<N; i++){
	x[i] = 1.0;
  }

  /* べき乗法の反復 */
  k = 0;
  while(1){

	/* 近似固有ベクトルの正規化（数値的安定化のため） */
	x_2 = norm_2(x);
	for(i=0; i<N; i++){
	  x[i] /= x_2;
	}

	/* 近似固有値と近似固有ベクトルの保存 */
	lam_old = lam;
	for(i=0; i<N; i++){
	  x_old[i] = x[i];
	}

	/* べき乗の計算 */
	mat_vec(x,A,x_old);

	/* 反復回数のカウント */
	k++;

	/* 固有値の計算（絶対値最大成分のところで比を計算） */
	max = fabs(x_old[0]);
	max_i = 0;
	for(i=1; i<N; i++){
	  if(max < fabs(x_old[i])){
		max = fabs(x_old[i]);
		max_i = i;
	  }
	}
	lam = x[max_i]/x_old[max_i];
	
	/* 固有値の計算（レイリー商により計算） */
	// mat_vec(y,A,x);
	// lam = dot(x,y)/dot(x,x);
	
	/* 収束判定 */
	if(fabs(lam - lam_old) < EPS * fabs(lam)){
	  printf(" *** 収束しました ***\n");
	  break;
	}else if(k == KMAX){
	  printf(" *** 最大反復回数に到達 ***\n");
	  break;
	}

  }
  
  /* 計算結果の出力 */
  printf("反復回数：%d\n", k);
  printf("近似固有値：%.16f\n", lam);
  printf("近似固有ベクトル：\n");
  for(i=0; i<N; i++){
	printf("x[%d] = %.16f\n", i+1, x[i]);
  }
  
  return 0;
}
