/*****************************/
/* 4章 常微分方程式の1段解法 */
/*****************************/

/* 本書の例題4.6に対するリチャードソンの補外法 */

#include<stdio.h>
#include<math.h>

#define M 6 // 検証する刻み幅の個数
#define R 4 // 補外の繰り返し数+1

/* 1階導関数 */
double f(double x, double y){
  return y;
}

/* 解析解 */
double y(double x){
  return M_E;
}

/* メイン関数 */
int main(void){
  int i, m, n, N;
  double h, x, x0, xn, Y, exact;
  double A[M][R];

  x0 = 0.0; // 始点
  xn = 1.0; // 終点
  
  /* 各刻み幅に対して検証 */
  N = 8;
  for(i=0; i<M; i++){

	h = (xn - x0)/N; // 刻み幅

	x = x0;  // 点の初期化
	Y = 1.0; // 初期条件

	/* オイラー法の計算 */
	for(n=0; n<N; n++){
	  Y += h*f(x,Y);
	  x += h;
	}

	/* オイラー法の結果を保存 */
	A[i][0] = Y;

	/* 分割数の更新 */
	N *= 2;

  }

  /* リチャードソンの補外法を適用 */
  for(m=1; m<M; m++){
	for(n=1; n<=fmin(m,R-1); n++){
	  A[m][n] = A[m][n-1] - (A[m-1][n-1] - A[m][n-1])/(pow(2,n) - 1);
	}
  }
  
  /* 近似解の出力 */
  printf("%6s: %18s: %18s: %18s: %18s:\n",
		 "h", "A_{m,0}", "A_{m,1}", "A_{m,2}", "A_{m,3}");
  for(i=0; i<87; i++){
	printf("-");
  }
  printf("\n");

  for(m=0; m<M; m++){
	printf("2^{-%d}: ", m+3);
	for(n=0; n<=fmin(m,R-1); n++){
	  printf("%.16f: ", A[m][n]);
	}
	printf("\n");
  }
  printf("\n");

  /* 誤差の出力 */
  printf("%6s: %11s: %11s: %11s: %11s:\n",
		 "h", "|A_{m,0}-e|", "|A_{m,1}-e|", "|A_{m,2}-e|", "|A_{m,3}-e|");
  for(i=0; i<59; i++){
	printf("-");
  }
  printf("\n");

  exact = y(1);
  for(m=0; m<M; m++){
	printf("2^{-%d}: ", m+3);
	for(n=0; n<=fmin(m,R-1); n++){
	  printf("   %.2e: ", fabs(A[m][n]-exact));
	}
	printf("\n");
  }
  
  return 0;
}
