/**************************/
/* 6章 数値積分と数値微分 */
/**************************/

/* 本書の例6.1に対するロンバーグ積分法 */

#include<stdio.h>
#include<math.h>

#define N 6 // 検証する分割の個数（p.74のNとは異なる）

/* 被積分関数 */
double f(double x){
  return sqrt(x);
}

/* メイン関数 */
int main(void){
  int m, k, n;
  double a, b, h, x, f_in, exact;
  double T[N][N];

  /* 真値の設定 */
  exact =  2.0*(pow(1.3,1.5)-1.0)/3.0; // 0.3214853684...

  
  /* 積分区間の設定 */
  a = 1.0;
  b = 1.3;

  printf("%2s: %18s: %8s\n", "n", "Romberg", "Error");
  printf("--------------------------------\n");
  
  /* 各分割数に対して台形則を適用（教科書のT_{m,0}に相当） */
  n = 2;
  for(m=0; m<N; m++){

	/* 刻み幅の設定 */
	h = (b-a)/n;

	/* 区間内側を計算 */
	x = a + h;
	f_in = f(x);
	for(k=2; k<n; k++){
	  x += h;
	  f_in += f(x);
	}

	/* 台形則の結果を保存 */
	T[m][0] = (f(a) + 2*f_in + f(b))*h/2.0;

	/* 分割数の更新 */
	n *= 2;

  }

  /* ロンバーグ積分法を適用：4段（n=2^4等分）まで */
  n = 2;
  for(m=1; m<4; m++){
	for(k=1; k<=m; k++){
	  T[m][k] = T[m][k-1] - (T[m-1][k-1] - T[m][k-1])/(pow(4,k) - 1);
	}
	n *= 2;
	printf("%2d: %.16f: %.2e\n", n, T[m][m], fabs(T[m][m]-exact));
  }
  
  return 0;
}
