/**************************/
/* 6章 数値積分と数値微分 */
/**************************/

/* 本書の例6.1に対するシンプソンの公式 */

#include<stdio.h>
#include<math.h>

#define N 6 // 検証する分割の個数

/* 被積分関数 */
double f(double x){
  return sqrt(x);
}

/* メイン関数 */
int main(void){
  int m, k, n;
  double a, b, h, h2, x, f_odd, f_eve, I, exact;

  /* 真値の設定 */
  exact =  2.0*(pow(1.3,1.5)-1.0)/3.0; // 0.3214853684...
  
  /* 積分区間の設定 */
  a = 1.0;
  b = 1.3;

  printf("%2s: %18s: %8s\n", "n", "Simpson", "Error");
  printf("--------------------------------\n");
  
  /* 各分割数に対して検証 */
  n = 2;
  for(m=1; m<=N; m++){

	/* 刻み幅の設定 */
	h2 = (b-a)/n;
	h = h2/2.0;

	/* 区間内側の奇数番目を計算 */
	x = a + h;
	f_odd = f(x);
	for(k=2; k<=n; k++){
	  x += h2;
	  f_odd += f(x);
	}

	/* 区間内側の偶数番目を計算 */
	x = a + h2;
	f_eve = f(x);
	for(k=2; k<n; k++){
	  x += h2;
	  f_eve += f(x);
	}

	/* 数値積分の結果 */
	I = (f(a) + 4*f_odd + 2*f_eve + f(b))*h/3.0;
	printf("%2d: %.16f: %.2e\n", n, I, fabs(I-exact));

	/* 分割数の更新 */
	n *= 2;

  }
  
  return 0;
}
