/****************************/
/* 8章 連立方程式の反復解法  */
/****************************/

/* 本書の演習問題8.2-(ii)に対するSOR法 */
/* 行列は2次元配列で与える（疎行列形式は使用しない） */

#include<stdio.h>
#include<math.h>

#define N 10           // 行列の次元
#define EPS 1.0e-9     // 許容誤差
#define MMAX 500       // 最大反復回数
#define OMG 1.56038792 // 緩和係数

/* ベクトルの∞-ノルムを求める関数 */
double norm_inf(double x[]){
  int i;
  double max;
  max = fabs(x[0]);
  for(i=1; i<N; i++){
	if(max < fabs(x[i])){
	  max = fabs(x[i]);
	}
  }
  return max;
}

/* メイン関数 */
int main(void){
  int i, j, m;
  double sum;

  double A[N][N]; // 係数行列（成分は以下で設定）
  //double b[N] = {6,10,8,10,7,6,7,10,7,5}; // 右辺ベクトル
  double b[N] = {1,-1,2,-1,-1,2,-2,2,-1,1}; // 右辺ベクトル
  //double b[N] = {2,4,-3,0,-1,4,-3,-1,4,-2}; // 右辺ベクトル
  //double b[N] = {-2,4,-1,-3,4,-1,0,-3,4,2}; // 右辺ベクトル
  double x[N];     // 近似解ベクトル
  double x_old[N]; // 更新前の近似解ベクトル
  double error[N]; // 近似解の差分を表す誤差ベクトル

  /* 係数行列の設定 */
  for(i=0; i<N; i++){
	for(j=0; j<N; j++){
	  if(i == j){
		A[i][j] = 2.0;
	  }else if(i == j+1){
		A[i][j] = -1.0;
	  }else if(i == j-1){
		A[i][j] = -1.0;
	  }else{
		A[i][j] = 0.0;
	  }
	}
  }

  /* 初期近似解の設定 */
  for(i=0; i<N; i++){
	x[i] = 0.0;
  }
  
  /* SOR法の反復 */
  m = 0;
  while(1){

	/* 近似解の保存 */
	for(i=0; i<N; i++){
	  x_old[i] = x[i];
	}

	/* 近似解の更新 */
	for(i=0; i<N; i++){
	  sum = 0.0;
	  for(j=0; j<N; j++){
		if(i != j){
		  sum += A[i][j]*x[j];
		}
	  }
	  sum = (b[i] - sum)/A[i][i];
	  x[i] = (1.0 - OMG)*x[i] + OMG*sum;
	}

	/* 反復回数のカウント */
	m++;

	/* 収束判定 */
	for(i=0; i<N; i++){
	  error[i] = x[i] - x_old[i];
	}
	if(norm_inf(error) < EPS * norm_inf(x_old)){
	  printf(" *** 収束しました ***\n");
	  break;
	}else if(m == MMAX){
	  printf(" *** 最大反復回数に到達 ***\n");
	  break;
	}

  }
  
  /* 計算結果の出力 */
  printf("反復回数：%d\n", m);
  printf("近似解ベクトル：\n");
  for(i=0; i<N; i++){
	printf("x[%d] = %.16f\n", i+1, x[i]);
  }
  
  return 0;
}
