# -*- coding: utf-8 -*-
"""**********************"""
""" 8章 連立方程式の反復解法 """
"""**********************"""

""" 本書の演習問題8.7[8.2-(ii)]に対するCG法（相対残差判定） """ 
""" 行列は2次元配列で与える（疎行列形式は使用しない） """

import numpy as np
import math

N    = 10     # 行列の次元
EPS  = 1.0e-9 # 許容誤差
MMAX = 30     # 最大反復回数

#""" 行列ベクトル積を計算する関数 """
#def mat_vec(y, A, x) :
#    for i in range(N) :
#        y[i] = 0
#        for j in range(N) :
#            y[i] += A[i,j]*x[j]

A = np.zeros((N,N)) # 係数行列（成分は以下で設定） 
#b = np.array([6.,10.,8.,10.,7.,6.,7.,10.,7.,5.]) # 右辺ベクトル
b = np.array([1.,-1.,2.,-1.,-1.,2.,-2.,2.,-1.,1.]) # 右辺ベクトル
#b = np.array([2.,4.,-3.,0.,-1.,4.,-3.,-1.,4.,-2.]) # 右辺ベクトル
#b = np.array([-2.,4.,-1.,-3.,4.,-1.,0.,-3.,4.,2.]) # 右辺ベクトル
x = np.zeros(N)     # 近似解ベクトル
r = np.zeros(N)     # 残差ベクトル
p = np.zeros(N)     # 探索方向ベクトル
w = np.zeros(N)     # 汎用ベクトル

""" 係数行列の設定 """
for i in range(N) :
    for j in range(N) :
        if i == j :
            A[i,j] = 2.0
        elif i == j+1 :
            A[i,j] = -1.0
        elif i == j -1 :
            A[i,j] = -1.0

""" 初期残差の計算  """
w = np.dot(A,x)
r = b - w

""" 初期探索方向の設定 """
p = np.copy(r)

""" 右辺ベクトルの2ノルムを保存 """
b_2 = math.sqrt(np.dot(b,b))

""" 残差の内積を保存 """
rho_old = np.dot(r,r)

""" CG法の反復 """
m = 0
while True :
    """ 係数alphaの計算 """
    w = np.dot(A,p)  
    # mat_vec(w, A, p) # C言語と合わせる場合 
    sigma = np.dot(p,w)
    alpha = rho_old/sigma

    """ 近似解の更新 """
    x = x + alpha*p

    """ 残差の更新 """
    r = r - alpha*w
    
    """ 反復回数のカウント """
    m += 1

    """ 収束判定 """
    if math.sqrt(np.dot(r,r)) < EPS*b_2 :
        print(" *** 収束しました ***")
        break
    elif m == MMAX :
        print(" *** 最大反復回数に到達 ***")
        break
    
    """ 係数betaの計算 """
    rho_new = np.dot(r,r)
    beta = rho_new/rho_old
    rho_old = rho_new

    """ 探索方向の更新  """
    p = r + beta*p

""" 計算結果の出力 """
print("反復回数：" + str(m))

print("近似解ベクトル：")
for i in range(N) :
    print("x[{:d}] = {:.16f}".format(i+1, x[i]))
