# -*- coding: utf-8 -*-
"""**********************"""
""" 8章 連立方程式の反復解法 """
"""**********************"""

""" 本書の演習問題8.7[8.2-(ii)]に対するCG法（相対残差判定） """ 
""" 疎行列形式（CRS形式）を使用 """

import numpy as np
import math

N    = 10     # 行列の次元
NNZ  = 28     # 非ゼロ要素数
EPS  = 1.0e-9 # 許容誤差
MMAX = 30     # 最大反復回数

""" 行列ベクトル積を計算する関数 """
def mat_vec(y, val, col_ind, row_ptr, x) :
    for i in range(N) :
        y[i] = 0
        for j in range(row_ptr[i],row_ptr[i+1]) :
            y[i] += val[j]*x[col_ind[j]]
            
""" 係数行列（CRS形式） """
val = np.array([2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0])
col_ind = np.array([0,1,0,1,2,1,2,3,2,3,4,3,4,5,4,5,6,5,6,7,6,7,8,7,8,9,8,9])
row_ptr = np.array([0,2,5,8,11,14,17,20,23,26,28])

b = np.array([1.,-1.,2.,-1.,-1.,2.,-2.,2.,-1.,1.]) # 右辺ベクトル
x = np.zeros(N)     # 近似解ベクトル
r = np.zeros(N)     # 残差ベクトル
p = np.zeros(N)     # 探索方向ベクトル
w = np.zeros(N)     # 汎用ベクトル

""" 初期残差の計算  """
mat_vec(w,val,col_ind,row_ptr,x)
r = b - w

""" 初期探索方向の設定 """
p = np.copy(r)

""" 右辺ベクトルの2ノルムを保存 """
b_2 = math.sqrt(np.dot(b,b))

""" 残差の内積を保存 """
rho_old = np.dot(r,r)

""" CG法の反復 """
m = 0
while True :
    """ 係数alphaの計算 """
    mat_vec(w,val,col_ind,row_ptr,p)
    sigma = np.dot(p,w)
    alpha = rho_old/sigma

    """ 近似解の更新 """
    x = x + alpha*p

    """ 残差の更新 """
    r = r - alpha*w
    
    """ 反復回数のカウント """
    m += 1

    """ 収束判定 """
    if math.sqrt(np.dot(r,r)) < EPS*b_2 :
        print(" *** 収束しました ***")
        break
    elif m == MMAX :
        print(" *** 最大反復回数に到達 ***")
        break
    
    """ 係数betaの計算 """
    rho_new = np.dot(r,r)
    beta = rho_new/rho_old
    rho_old = rho_new

    """ 探索方向の更新  """
    p = r + beta*p

""" 計算結果の出力 """
print("反復回数：" + str(m))

print("近似解ベクトル：")
for i in range(N) :
    print("x[{:d}] = {:.16f}".format(i+1, x[i]))
