# -*- coding: utf-8 -*-
"""**********************"""
""" 8章 連立方程式の反復解法 """
"""**********************"""

""" 本書の演習問題8.2-(ii)に対するJacobi法 """ 
""" 行列は2次元配列で与える（疎行列形式は使用しない）""" 

import numpy as np

N    = 10     # 行列の次元
EPS  = 1.0e-9 # 許容誤差
MMAX = 500    # 最大反復回数

A = np.zeros((N,N)) # 係数行列（成分は以下で設定） 
#b = np.array([6.,10.,8.,10.,7.,6.,7.,10.,7.,5.]) # 右辺ベクトル
b = np.array([1.,-1.,2.,-1.,-1.,2.,-2.,2.,-1.,1.]) # 右辺ベクトル
#b = np.array([2.,4.,-3.,0.,-1.,4.,-3.,-1.,4.,-2.]) # 右辺ベクトル
#b = np.array([-2.,4.,-1.,-3.,4.,-1.,0.,-3.,4.,2.]) # 右辺ベクトル
x     = np.zeros(N)     # 近似解ベクトル
x_old = np.zeros(N)     # 更新前の近似解ベクトル
error = np.zeros(N)     # 近似解の差分を表す誤差ベクトル

""" 係数行列の設定 """
for i in range(N) :
    for j in range(N) :
        if i == j :
            A[i,j] = 2.0
        elif i == j+1 :
            A[i,j] = -1.0
        elif i == j -1 :
            A[i,j] = -1.0

""" Jacobi法の反復 """
m = 0
while True :
    """ 近似解の保存 """
    x_old = np.copy(x)

    """ 近似解の更新 """
    for i in range(N) :
        sum = 0
        for j in range(N) :
            if i != j :
                sum += A[i,j]*x_old[j]
        x[i] = (b[i] - sum)/A[i,i]
    
    """ 反復回数のカウント """
    m += 1

    """ 収束判定 """
    error = x - x_old
    if max(abs(error)) < EPS*max(abs(x_old)) :
        print(" *** 収束しました ***")
        break
    elif m == MMAX :
        print(" *** 最大反復回数に到達 ***")
        break

""" 計算結果の出力 """
print("反復回数：" + str(m))

print("近似解ベクトル：")
for i in range(N) :
    print("x[{:d}] = {:.16f}".format(i+1, x[i]))
