# -*- coding: utf-8 -*-
"""***************"""
""" 9章 固有値の計算 """
"""***************"""

""" 本書の例題9.1に対するJacobi法 """ 
""" 固定回数（収束判定は設定していない） """

import numpy as np
import math

N    = 3  # 行列の次元
ITER = 3  # 反復回数

A = np.array([[3, 0.01, 0.1],\
              [0.01, 2, 0.01],\
              [0.1, 0.01, 1]])

print("A = ")
np.set_printoptions(formatter={"float": "{:.16f}".format}) # 行列成分の表示形式を%.16fに揃える
print(A)

""" 固有ベクトル行列の初期化 """
P = np.eye(N) 
A1= np.copy(A)
P1= np.copy(P)

""" ヤコビ法の反復 """
q = 0
while q < ITER :
    print("-"*62)
    
    """ ステップ1 """
    max = 0.0
    for i in range(N-1) :
        for j in range(i+1,N) :
            if abs(A[i,j]) > max :
                max = abs(A[i,j])
                k = i
                m = j
    print("k = {:d}, m = {:d}".format(k, m))

    """ ステップ2 """
    u = (A[k,k] - A[m,m])/2.0
    v = math.sqrt(u*u + A[k,m]*A[k,m])
    if u >= 0 :
        c = math.sqrt((1+u/v)/2.0)
        s = A[k,m]/(2.0*v*c)
    else :
        c = math.sqrt((1-u/v)/2.0)
        s = -A[k,m]/(2.0*v*c)

    print("cos = {:.16f}, sin = {:.16f}\n".format(c, s))
    
    """ ステップ3,4 （回転行列は陽に作らない） """
    for i in range(N) :

        if i == k : # 第k行の計算
            for j in range(i,N) : # 上三角部分のみ計算
                if j == k :
                    A1[i,j] = c*c*A[k,k] + 2.0*s*c*A[k,m] + s*s*A[m,m]
                elif j == m :
                    A1[i,j] = 0.0
                else :
                    A1[i,j] = c*A[k,j] + s*A[m,j]
       
        elif i == m : # 第m行の計算
            for j in range(i,N) : # 上三角部分のみ計算
                if j == k :
                    A1[i,j] = 0.0
                elif j == m :
                    A1[i,j] = s*s*A[k,k] - 2.0*s*c*A[k,m] + c*c*A[m,m]
                else :
                    A1[i,j] = -s*A[k,j] + c*A[m,j]
       
        else : # その他の行の計算 
            for j in range(i,N) : # 上三角部分のみ計算
                if j == k :
                    A1[i,j] = c*A[i,k] + s*A[i,m]
                elif j == m :
                    A1[i,j] = -s*A[i,k] + c*A[i,m]
                else :
                    A1[i,j] = A[i,j]
        
        """ 狭義下三角部分にコピー """
        for i in range(N-1) :
            for j in range(i+1,N) :
                A1[j,i] = A1[i,j]
        
        """ 固有ベクトル行列の更新 """
        for i in range(N) :
            for j in range(N) :
                if j == k :
                    P1[i,j] = c*P[i,k] + s*P[i,m]
                elif j == m :
                    P1[i,j] = -s*P[i,k] + c*P[i,m]
                else :
                    P1[i,j] = P[i,j]

    """ 反復回数のカウント """
    q += 1

    """ 更新後の行列の表示 """
    print("A_{:d} = ".format(q))
    print(A1)
    print("")
    print("P_0...P_{:d} = ".format(q-1))
    print(P1)
    print("")

    """ 行列の入れ替え """
    A = np.copy(A1)
    P = np.copy(P1)