# -*- coding: utf-8 -*-
"""**********************"""
""" 8章 連立方程式の反復解法 """
"""**********************"""

""" 本書の演習問題8.2-(ii)に対するJacobi法 """ 
""" 疎行列形式（CRS形式）を使用 """

import numpy as np

N    = 10     # 行列の次元
NNZ  = 28     # 非ゼロ要素数
EPS  = 1.0e-9 # 許容誤差
MMAX = 500    # 最大反復回数

""" CRS形式の対角成分を取り出す関数 """
def mat_diag(daiag, val, cod_ind, row_ptr) :
    for i in range(N) :
        for j in range(row_ptr[i],row_ptr[i+1]) :
            if i == col_ind[j] :
                diag[i] = val[j]


""" 係数行列（CRS形式） """
val = np.array([2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0,-1.0,-1.0,2.0])
col_ind = np.array([0,1,0,1,2,1,2,3,2,3,4,3,4,5,4,5,6,5,6,7,6,7,8,7,8,9,8,9])
row_ptr = np.array([0,2,5,8,11,14,17,20,23,26,28])

b = np.array([1.,-1.,2.,-1.,-1.,2.,-2.,2.,-1.,1.]) # 右辺ベクトル
diag  = np.zeros(N)     # 対角成分
x     = np.zeros(N)     # 近似解ベクトル
x_old = np.zeros(N)     # 更新前の近似解ベクトル
error = np.zeros(N)     # 近似解の差分を表す誤差ベクトル

""" 対角成分の取り出し """
mat_diag(diag, val, col_ind, row_ptr)

""" Jacobi法の反復 """
m = 0
while True :
    """ 近似解の保存 """
    x_old = np.copy(x)

    """ 近似解の更新 """
    for i in range(N) :
        sum = 0
        for j in range(row_ptr[i],row_ptr[i+1]) :
            if i != col_ind[j] :
                sum += val[j]*x_old[col_ind[j]]
        x[i] = (b[i] - sum)/diag[i]

    """ 反復回数のカウント """
    m += 1

    """ 収束判定 """
    error = x - x_old
    if max(abs(error)) < EPS*max(abs(x_old)) :
        print(" *** 収束しました ***")
        break
    elif m == MMAX :
        print(" *** 最大反復回数に到達 ***")
        break

""" 計算結果の出力 """
print("反復回数：" + str(m))

print("近似解ベクトル：")
for i in range(N) :
    print("x[{:d}] = {:.16f}".format(i+1, x[i]))