# -*- coding: utf-8 -*-
"""***************************"""
""" 3章 非線形方程式に対する反復法 """
"""***************************"""

"""本書の例3.2に対するニュートン法（相対誤差判定）""" 

import math

EPS  = 1.0e-12 #許容誤差
NMAX = 20      #最大反復回数

""" 関数値の計算 """
def f(x) :
    return x - 1.0/math.exp(x)

""" 微分係数の計算 """
def df(x) :
    return 1.0 + 1.0/math.exp(x) 

n = 0    # 反復回数
x = 0.5  # 初期近似解の設定

print(" n: x_n") 
print("----------------------")
print("{:2d}: {:.16f}".format(n, x)) 

""" ニュートン反復 """
while True :

    """ 近似解の保存 """ 
    x_old = x 

    """ 近似解の更新 """ 
    x = x - f(x)/df(x) 

    """ 反復回数のカウント """ 
    n += 1

    """ 近似解の出力 """ 
    print("{:2d}: {:.16f}".format(n, x))

    """ 収束判定 """ 
    if abs(x-x_old) < EPS * abs(x_old): 
        print(" *** 収束しました ***")
        break
    elif n == NMAX :
        print(" *** 最大反復回数に到達 ***") 
        break
        