# -*- coding: utf-8 -*-
"""**********************"""
""" 8章 連立方程式の反復解法 """
"""**********************"""

""" 本書の例題8.2に対する連立ニュートン法（残差判定） """ 

import numpy as np
import math

N    = 2       # 行列の次元
EPS  = 1.0e-10 # 許容誤差
KMAX = 20      # 最大反復回数
A    = 0.7     # 問題の係数A
B    = 0.2     # 問題の係数B

""" 関数値の計算 """
def function(f, x) :
    f[0] = x[0] - A*math.sin(x[0]) - B*math.cos(x[1])
    f[1] = x[1] - A*math.cos(x[0]) - B*math.sin(x[1])

""" ヤコビ行列の計算 """
def jacobian(J, x) :
    J[0, 0] = 1.0 - A*math.cos(x[0])
    J[0, 1] = B*math.sin(x[1])
    J[1, 0] = A*math.sin(x[0])
    J[1, 1] = 1.0 - B*math.cos(x[1])


#""" ニュートン法の更新量（連立1次方程式の求解） """
#def newton_step(J, d, f) :
#    """ Gaussの消去法（ピボット選択なし） """
#    for k in range(N-1) :
#        for i in range(k+1,N) :
#            alpha = J[i,k]/J[k,k]
#            J[i,k+1:N] -= alpha*J[k,k+1:N] 
#            f[i] -= alpha*f[k]
#    d[N-1] = f[N-1]/J[N-1,N-1]
#    for i in reversed(range(N-1)) :
#        d[i] = (f[i] - np.dot(J[i,i+1:N],d[i+1:N]))/J[i,i]

J = np.zeros((N,N)) # ヤコビ行列
x = np.zeros(N)     # 近似解ベクトル
f = np.zeros(N)     # 残差ベクトル
d = np.zeros(N)     # ニュートン法の更新量を表すベクトル

""" 初期近似解の設定 """
x = np.zeros(N)

print(" k:"+" "*16 + "x_k:" + " "*16 + "y_k")
print("-"*42)

""" ニュートン反復 """
k = 0
while True :
    """ 近似解の出力 """
    print("{:2d}: {:.16f}: {:.16f}".format(k, x[0], x[1]))

    """ 関数値（残差）の計算 """
    function(f, x)

    """ 収束判定 """
    if math.sqrt(np.dot(f,f)) < EPS :
        print(" *** 収束しました ***")
        break
    elif k == KMAX :
        print(" *** 最大反復回数に到達 ***")
        break
    
    """ ヤコビ行列の計算 """
    jacobian(J, x)

    """ ニュートン法の更新量の計算 """
    d = np.linalg.solve(J, -f) # ライブラリを用いて方程式を解く場合
    #newton_step(J, d, -f) # C言語と同様に、ガウスの消去法（自作）を用いて方程式を解く場合

    """ 近似解の更新 """
    x = x + d

    """ 反復回数のカウント """
    k += 1