# -*- coding: utf-8 -*-
"""**************************"""
""" 2章 連立1次方程式の直接解法  """
"""**************************"""

""" 本書の演習問題2.1に対する部分ピボット選択付きGaussの消去法 """ 

import numpy as np

N = 4          # 行列の次元
EPS  = 1.0e-15 # ピボットの許容スケール

A = np.array([[1., 2., -12., 8.],\
              [5., 4., 7., -2.],\
              [-3., 7., 9., 5.],\
              [6., -12., -9., 3.]]) # 係数行列

b = np.array([27., 4., 11., 49.]) # 右辺ベクトル
x = np.empty(N) # 解ベクトル

""" 前進消去過程 """
for k in range(N-1) :

    """ 部分ピボット選択 """
    pivot = max(abs(A[k:N,k])) 
    sigma = abs(A[:,k]).argmax()
     
    if pivot < EPS :
        print("ピボットの値が許容スケール未満です")
        print(" *** 計算を強制終了します ***")
        break
    if k < sigma : # k行目とsigma行目の交換
        A[[k,sigma],:] = A[[sigma,k],:]
        b[k], b[sigma] = b[sigma], b[k]

    """ 掃き出し """
    for i in range(k+1,N) :
        alpha = A[i,k]/A[k,k]
        A[i,k+1:N] -= alpha*A[k,k+1:N] 
        b[i] -= alpha*b[k]

""" 後退代入過程 """
x[N-1] = b[N-1]/A[N-1,N-1]
for i in reversed(range(N-1)) :
    x[i] = (b[i] - np.dot(A[i,i+1:N],x[i+1:N]))/A[i,i]


""" 近似解の出力 """
for i in range(N) :
    print("x[{:d}] = {:.16f}".format(i+1, x[i]))

#""" 近似解の出力(別形式) """
#print("x = ")
#np.set_printoptions(formatter={"float": "{:.16f}".format})
#print(x)
