# -*- coding: utf-8 -*-
"""****************"""
""" 9章 固有値の計算 """
"""****************"""

""" 本書の演習問題9.1-(ii)に対するべき乗法（相対誤差判定） """ 

import numpy as np
import math

N    = 3      # 行列の次元
EPS  = 1.0e-9 # 許容誤差
KMAX = 30     # 最大反復回数

A = np.array([[2., -1., 0],\
              [-1., 2., -1],\
              [0, -1., 2.]])

""" 初期値の設定 """
lam = 0.0 
x = np.ones(N)     # 近似固有ベクトル
x_old = np.copy(x) # 更新前の近似固有ベクトル

""" べき乗法の反復 """
k = 0
while True :
    """ 近似固有ベクトルの正規化（数値的安定化のため） """
    x = x/math.sqrt(np.dot(x,x))

    """ 近似固有値と近似固有ベクトルの保存 """
    lam_old = lam
    x_old = np.copy(x)

    """ べき乗の計算 """
    x = np.dot(A,x)

    """ 反復回数のカウント """
    k += 1

    """ 固有値の計算（絶対値最大成分のところで比を計算）"""
    max_i = abs(x).argmax()
    lam = x[max_i]/x_old[max_i]

    """ 固有値の計算（レイリー商により計算）"""
    # lam = np.dot(x,np.dot(A,x))/np.dot(x,x) 
    
    """ 収束判定 """
    if abs(lam - lam_old) < EPS * abs(lam) :
       print(" *** 収束しました ***")
       break
    elif k == KMAX :
        print(" *** 最大反復回数に到達 ***")
        break

""" 計算結果の出力 """
print("反復回数：" + str(k))
print("近似固有値：{:.16f}".format(lam))
print("近似固有ベクトル：")
for i in range(N) :
    print("x[{:d}] = {:.16f}".format(i+1, x[i]))