# -*- coding: utf-8 -*-
"""************************"""
""" 4章 常微分方程式の1段解法 """
"""************************"""

""" 本書の例題4.6に対するリチャードソンの補外法 """ 

import math
M = 6 # 検証する刻み幅の個数
R = 4 # 補外の繰り返し数+1

""" 1階導関数 """
def f (x, y) :
    return y
   
""" 解析解 """
def y (x) :
    return math.e

A  = [[0 for i in range(R)] for j in range(M)]
x0 = 0.0 # 始点
xn = 1.0 # 終点

""" 各刻み幅に対して検証 """
N = 8
for i in range(M) :
    h = (xn -x0)/N # 刻み幅

    x = x0  # 点の初期化
    Y = 1.0 # 初期条件
    
    """ オイラー法の計算 """
    for n in range(N) :
       Y += h*f(x,Y)
       x += h
    
    """ オイラー法の結果を保存 """
    A[i][0] = Y

    """ 分割数の更新 """
    N *= 2

""" リチャードソンの補外法を適用 """
for m in range(1,M) :
    for n in range(1,min(m+1,R)) :
        A[m][n] = A[m][n-1] - (A[m-1][n-1] - A[m][n-1])/(2**n - 1)


""" 近似解の出力 """
print("h:".rjust(7) + "A_{m,0}:".rjust(20) + \
        "A_{m,1}:".rjust(20) + "A_{m,2}:".rjust(20) +"A_{m,3}:".rjust(20))
print("-"*87)

for m in range(M)  :
    print("2^{-"+"{:d}".format(m+3)+"}:", end='')
    for n in range(min(m+1,R)) :
        print(" {:.16f}:".format(A[m][n]), end='')
    print("")
print("")

print("h:".rjust(7) + " |A_{m,0}-e|:" + \
        "  |A_{m,1}-e|:" + "  |A_{m,2}-e|:" + "  |A_{m,3}-e|:")
print("-"*62)

exact = y(1)
for m in range(M)  :
    print("2^{-"+"{:d}".format(m+3)+"}:", end='')
    for n in range(min(m+1,R)) :
        print("    {:.2e}: ".format(abs(A[m][n]-exact)), end='')
    print("")
