# -*- coding: utf-8 -*-
"""**********************"""
""" 6章 数値積分と数値微分 """
"""**********************"""

""" 本書の例6.1に対するロンバーグ積分法 """ 

import numpy as np

N = 6 # 検証する分割の個数（p.74のNとは異なる）

""" 被積分関数 """
def f(x) :
    return x**0.5
    
""" 真値の設定 """
exact = 2.0*(1.3**1.5-1.0)/3.0 # 0.3214853684...

""" 積分区間の設定 """
a = 1.0
b = 1.3
T = np.empty((N,N))
print(" n: "+"Romberg: ".rjust(20) + "Error".rjust(8))
print("-"*32)

""" 各分割数に対して台形則を適用（教科書のT_{m,0}に相当） """
n = 2
for m in range(N) :
    
    """ 刻み幅の設定 """
    h = (b-a)/n

    """ 区間内側を計算  """
    x = a + h
    f_in = f(x)
    for k in range(2,n) :
        x += h
        f_in += f(x)

    """ 台形則の結果を保存 """
    T[m][0] = (f(a) + 2*f_in + f(b))*h/2.0

    """ 分割数の更新 """
    n *= 2

""" ロンバーグ積分法を適用:4段（n=2^4等分）まで """
n = 2
for m in range(1,4) :
    for k in range(1,m+1) :
        T[m,k] = T[m,k-1] - (T[m-1,k-1] - T[m,k-1])/(pow(4,k) - 1)
        
    n *= 2
    print("{:2d}: {:.16f}: {:.2e}".format( n, T[m][m], abs(T[m][m]-exact)))
